@@ -345,8 +345,8 @@ static void playback_handle_data(SpiceChannel *channel, SpiceMsgIn *in)
     SpiceMsgPlaybackPacket *packet = spice_msg_in_parsed(in);
 
 #ifdef DEBUG
-    SPICE_DEBUG("%s: time %d data %p size %d", __FUNCTION__,
-            packet->time, packet->data, packet->data_size);
+    CHANNEL_DEBUG(channel, "%s: time %d data %p size %d", __FUNCTION__,
+                  packet->time, packet->data, packet->data_size);
 #endif
 
     if (c->last_time > packet->time)
@@ -390,8 +390,8 @@ static void playback_handle_mode(SpiceChannel *channel, SpiceMsgIn *in)
     SpicePlaybackChannelPrivate *c = SPICE_PLAYBACK_CHANNEL(channel)->priv;
     SpiceMsgPlaybackMode *mode = spice_msg_in_parsed(in);
 
-    SPICE_DEBUG("%s: time %d mode %d data %p size %d", __FUNCTION__,
-            mode->time, mode->mode, mode->data, mode->data_size);
+    CHANNEL_DEBUG(channel, "%s: time %d mode %d data %p size %d", __FUNCTION__,
+                  mode->time, mode->mode, mode->data, mode->data_size);
 
     c->mode = mode->mode;
     switch (c->mode) {
@@ -411,8 +411,8 @@ static void playback_handle_start(SpiceChannel *channel, SpiceMsgIn *in)
     SpiceMsgPlaybackStart *start = spice_msg_in_parsed(in);
     int celt_mode_err;
 
-    SPICE_DEBUG("%s: fmt %d channels %d freq %d time %d", __FUNCTION__,
-            start->format, start->channels, start->frequency, start->time);
+    CHANNEL_DEBUG(channel, "%s: fmt %d channels %d freq %d time %d", __FUNCTION__,
+                  start->format, start->channels, start->frequency, start->time);
 
     c->frame_count = 0;
     c->last_time = start->time;
@@ -509,7 +509,7 @@ void spice_playback_channel_set_delay(SpicePlaybackChannel *channel, guint32 del
 
     g_return_if_fail(SPICE_IS_PLAYBACK_CHANNEL(channel));
 
-    SPICE_DEBUG("playback set_delay %u ms", delay_ms);
+    CHANNEL_DEBUG(channel, "playback set_delay %u ms", delay_ms);
 
     c = channel->priv;
     spice_session_set_mm_time(spice_channel_get_session(SPICE_CHANNEL(channel)),
