@@ -443,8 +443,8 @@ static void record_handle_start(SpiceChannel *channel, SpiceMsgIn *in)
     SpiceRecordChannelPrivate *c = SPICE_RECORD_CHANNEL(channel)->priv;
     SpiceMsgRecordStart *start = spice_msg_in_parsed(in);
 
-    SPICE_DEBUG("%s: fmt %d channels %d freq %d", __FUNCTION__,
-            start->format, start->channels, start->frequency);
+    CHANNEL_DEBUG(channel, "%s: fmt %d channels %d freq %d", __FUNCTION__,
+                  start->format, start->channels, start->frequency);
 
     c->frame_bytes = FRAME_SIZE * 16 * start->channels / 8;
 
