@@ -100,9 +100,11 @@ enum {
     PROP_RESIZE_GUEST,
     PROP_AUTO_CLIPBOARD,
     PROP_SCALING,
+    PROP_ONLY_DOWNSCALE,
     PROP_DISABLE_INPUTS,
     PROP_ZOOM_LEVEL,
     PROP_MONITOR_ID,
+    PROP_KEYPRESS_DELAY,
     PROP_READY
 };
 
@@ -169,16 +171,22 @@ static void spice_display_get_property(GObject    *object,
         break;
     case PROP_SCALING:
         g_value_set_boolean(value, d->allow_scaling);
-	break;
+        break;
+    case PROP_ONLY_DOWNSCALE:
+        g_value_set_boolean(value, d->only_downscale);
+        break;
     case PROP_DISABLE_INPUTS:
         g_value_set_boolean(value, d->disable_inputs);
-	break;
+        break;
     case PROP_ZOOM_LEVEL:
         g_value_set_int(value, d->zoom_level);
         break;
     case PROP_READY:
         g_value_set_boolean(value, d->ready);
         break;
+    case PROP_KEYPRESS_DELAY:
+        g_value_set_uint(value, d->keypress_delay);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -192,9 +200,7 @@ static void scaling_updated(SpiceDisplay *display)
 
     recalc_geometry(GTK_WIDGET(display));
     if (d->ximage && window) { /* if not yet shown */
-        int ww, wh;
-        gdk_drawable_get_size(gtk_widget_get_window(GTK_WIDGET(display)), &ww, &wh);
-        gtk_widget_queue_draw_area(GTK_WIDGET(display), 0, 0, ww, wh);
+        gtk_widget_queue_draw(GTK_WIDGET(display));
     }
 }
 
@@ -333,6 +339,10 @@ static void spice_display_set_property(GObject      *object,
         d->allow_scaling = g_value_get_boolean(value);
         scaling_updated(display);
         break;
+    case PROP_ONLY_DOWNSCALE:
+        d->only_downscale = g_value_get_boolean(value);
+        scaling_updated(display);
+        break;
     case PROP_AUTO_CLIPBOARD:
         g_object_set(d->gtk_session, "auto-clipboard",
                      g_value_get_boolean(value), NULL);
@@ -347,6 +357,9 @@ static void spice_display_set_property(GObject      *object,
         d->zoom_level = g_value_get_int(value);
         scaling_updated(display);
         break;
+    case PROP_KEYPRESS_DELAY:
+        d->keypress_delay = g_value_get_uint(value);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -372,6 +385,7 @@ static void session_inhibit_keyboard_grab_changed(GObject    *gobject,
     g_object_get(d->session, "inhibit-keyboard-grab",
                  &d->keyboard_grab_inhibit, NULL);
     update_keyboard_grab(display);
+    update_mouse_grab(display);
 }
 
 static void spice_display_dispose(GObject *obj)
@@ -385,6 +399,11 @@ static void spice_display_dispose(GObject *obj)
     g_clear_object(&d->session);
     d->gtk_session = NULL;
 
+    if (d->key_delayed_id) {
+        g_source_remove(d->key_delayed_id);
+        d->key_delayed_id = 0;
+    }
+
     G_OBJECT_CLASS(spice_display_parent_class)->dispose(obj);
 }
 
@@ -844,7 +863,9 @@ static void update_mouse_grab(SpiceDisplay *display)
 {
     SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(display);
 
-    if (d->mouse_grab_enable && !d->disable_inputs)
+    if (d->mouse_grab_enable &&
+        !d->keyboard_grab_inhibit &&
+        !d->disable_inputs)
         try_mouse_grab(display);
     else
         try_mouse_ungrab(display);
@@ -869,19 +890,12 @@ static void recalc_geometry(GtkWidget *widget)
     SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(display);
     gdouble zoom = 1.0;
 
-    d->mx = 0;
-    d->my = 0;
-    if (!spicex_is_scaled(display)) {
-        if (d->ww > d->area.width)
-            d->mx = (d->ww - d->area.width) / 2;
-        if (d->wh > d->area.height)
-            d->my = (d->wh - d->area.height) / 2;
-    } else
+    if (spicex_is_scaled(display))
         zoom = (gdouble)d->zoom_level / 100;
 
-    SPICE_DEBUG("recalc geom monitor: %d:%d, guest +%d+%d:%dx%d, window %dx%d, zoom %g, offset +%d+%d",
+    SPICE_DEBUG("recalc geom monitor: %d:%d, guest +%d+%d:%dx%d, window %dx%d, zoom %g",
                 d->channel_id, d->monitor_id, d->area.x, d->area.y, d->area.width, d->area.height,
-                d->ww, d->wh, zoom, d->mx, d->my);
+                d->ww, d->wh, zoom);
 
     if (d->resize_guest_enable)
         spice_main_set_display(d->main, get_display_id(display),
@@ -979,8 +993,46 @@ static gboolean expose_event(GtkWidget *widget, GdkEventExpose *expose)
 #endif
 
 /* ---------------------------------------------------------------- */
+typedef enum {
+    SEND_KEY_PRESS,
+    SEND_KEY_RELEASE,
+} SendKeyType;
 
-static void send_key(SpiceDisplay *display, int scancode, int down)
+static void key_press_and_release(SpiceDisplay *display)
+{
+    SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(display);
+
+    if (d->key_delayed_scancode == 0)
+        return;
+
+    spice_inputs_key_press_and_release(d->inputs, d->key_delayed_scancode);
+    d->key_delayed_scancode = 0;
+
+    if (d->key_delayed_id) {
+        g_source_remove(d->key_delayed_id);
+        d->key_delayed_id = 0;
+    }
+}
+
+static gboolean key_press_delayed(gpointer data)
+{
+    SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(data);
+
+    if (d->key_delayed_scancode == 0)
+        return FALSE;
+
+    spice_inputs_key_press(d->inputs, d->key_delayed_scancode);
+    d->key_delayed_scancode = 0;
+
+    if (d->key_delayed_id) {
+        g_source_remove(d->key_delayed_id);
+        d->key_delayed_id = 0;
+    }
+
+    return FALSE;
+}
+
+static void send_key(SpiceDisplay *display, int scancode, SendKeyType type, gboolean press_delayed)
 {
     SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(display);
     uint32_t i, b, m;
@@ -996,15 +1048,40 @@ static void send_key(SpiceDisplay *display, int scancode, int down)
     m = (1 << b);
     g_return_if_fail(i < SPICE_N_ELEMENTS(d->key_state));
 
-    if (down) {
-        spice_inputs_key_press(d->inputs, scancode);
+    switch (type) {
+    case SEND_KEY_PRESS:
+        /* ensure delayed key is pressed before any new input event */
+        key_press_delayed(display);
+
+        if (press_delayed &&
+            d->keypress_delay != 0 &&
+            !(d->key_state[i] & m)) {
+            g_warn_if_fail(d->key_delayed_id == 0);
+            d->key_delayed_id = g_timeout_add(d->keypress_delay, key_press_delayed, display);
+            d->key_delayed_scancode = scancode;
+        } else
+            spice_inputs_key_press(d->inputs, scancode);
+
         d->key_state[i] |= m;
-    } else {
-        if (!(d->key_state[i] & m)) {
-            return;
+        break;
+
+    case SEND_KEY_RELEASE:
+        if (!(d->key_state[i] & m))
+            break;
+
+        if (d->key_delayed_scancode == scancode)
+            key_press_and_release(display);
+        else {
+            /* ensure delayed key is pressed before other key are released */
+            key_press_delayed(display);
+            spice_inputs_key_release(d->inputs, scancode);
         }
-        spice_inputs_key_release(d->inputs, scancode);
+
         d->key_state[i] &= ~m;
+        break;
+
+    default:
+        g_warn_if_reached();
     }
 }
 
@@ -1019,7 +1096,7 @@ static void release_keys(SpiceDisplay *display)
             continue;
         }
         for (b = 0; b < 32; b++) {
-            send_key(display, i * 32 + b, 0);
+            send_key(display, i * 32 + b, SEND_KEY_RELEASE, FALSE);
         }
     }
 }
@@ -1062,9 +1139,10 @@ static gboolean key_event(GtkWidget *widget, GdkEventKey *key)
     SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(display);
     int scancode;
 
-    SPICE_DEBUG("%s %s: keycode: %d  state: %d  group %d",
+
+    SPICE_DEBUG("%s %s: keycode: %d  state: %d  group %d modifier %d",
             __FUNCTION__, key->type == GDK_KEY_PRESS ? "press" : "release",
-            key->hardware_keycode, key->state, key->group);
+            key->hardware_keycode, key->state, key->group, key->is_modifier);
 
     if (check_for_grab_key(display, key->type, key->keyval)) {
         g_signal_emit(widget, signals[SPICE_DISPLAY_GRAB_KEY_PRESSED], 0);
@@ -1089,10 +1167,10 @@ static gboolean key_event(GtkWidget *widget, GdkEventKey *key)
                                             key->hardware_keycode);
     switch (key->type) {
     case GDK_KEY_PRESS:
-        send_key(display, scancode, 1);
+        send_key(display, scancode, SEND_KEY_PRESS, !key->is_modifier);
         break;
     case GDK_KEY_RELEASE:
-        send_key(display, scancode, 0);
+        send_key(display, scancode, SEND_KEY_RELEASE, !key->is_modifier);
         break;
     default:
         g_warn_if_reached();
@@ -1131,12 +1209,12 @@ void spice_display_send_keys(SpiceDisplay *display, const guint *keyvals,
 
     if (kind & SPICE_DISPLAY_KEY_EVENT_PRESS) {
         for (i = 0 ; i < nkeyvals ; i++)
-            send_key(display, get_scancode_from_keyval(display, keyvals[i]), 1);
+            send_key(display, get_scancode_from_keyval(display, keyvals[i]), SEND_KEY_PRESS, FALSE);
     }
 
     if (kind & SPICE_DISPLAY_KEY_EVENT_RELEASE) {
         for (i = (nkeyvals-1) ; i >= 0 ; i--)
-            send_key(display, get_scancode_from_keyval(display, keyvals[i]), 0);
+            send_key(display, get_scancode_from_keyval(display, keyvals[i]), SEND_KEY_RELEASE, FALSE);
     }
 }
 
@@ -1241,63 +1319,76 @@ static int button_mask_gdk_to_spice(int gdk)
     return spice;
 }
 
+G_GNUC_INTERNAL
+void spicex_transform_input (SpiceDisplay *display,
+                             double window_x, double window_y,
+                             int *input_x, int *input_y)
+{
+    SpiceDisplayPrivate *d = display->priv;
+    int display_x, display_y, display_w, display_h;
+    double is;
+
+    spice_display_get_scaling(display, NULL,
+                              &display_x, &display_y,
+                              &display_w, &display_h);
+
+    /* For input we need a different scaling factor in order to
+       be able to reach the full width of a display. For instance, consider
+       a display of 100 pixels showing in a window 10 pixels wide. The normal
+       scaling factor here would be 100/10==10, but if you then take the largest
+       possible window coordinate, i.e. 9 and multiply by 10 you get 90, not 99,
+       which is the max display coord.
+
+       If you want to be able to reach the last pixel in the window you need
+       max_window_x * input_scale == max_display_x, which is
+       (window_width - 1) * input_scale == (display_width - 1)
+
+       Note, this is the inverse of s (i.e. s ~= 1/is) as we're converting the
+       coordinates in the inverse direction (window -> display) as the fb size
+       (display -> window).
+    */
+    is = (double)(d->area.width-1) / (double)(display_w-1);
+
+    window_x -= display_x;
+    window_y -= display_y;
+
+    *input_x = floor (window_x * is);
+    *input_y = floor (window_y * is);
+}
+
 static gboolean motion_event(GtkWidget *widget, GdkEventMotion *motion)
 {
     SpiceDisplay *display = SPICE_DISPLAY(widget);
     SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(display);
-    int ww, wh;
+    int x, y;
 
     if (!d->inputs)
         return true;
     if (d->disable_inputs)
         return true;
 
-    gdk_drawable_get_size(gtk_widget_get_window(widget), &ww, &wh);
-    if (spicex_is_scaled(display) && (d->area.width != ww || d->area.height != wh)) {
-        double sx, sy;
-        sx = (double)d->area.width / (double)ww;
-        sy = (double)d->area.height / (double)wh;
-
-        /* Scaling the desktop, so scale the mouse coords by the same.
-         * Ratio for the rounding used:
-         * 1) We should be able to send mouse coords for all 4 corner pixels
-         * 2) The GdkMotion events are double's, so we've X.Xfrac and Y.Yfrac
-         * 3) The X and Y integer values always go from 0 - (ww/wh - 1)
-         * 4) Note that even if Xfrac = .99999, X will still go from 0 - (ww-1)
-         *    so x will go from 0.99999 - (ww-1).99999
-         * 5) Given 4, the only way to be sure we can always generate 0
-         *    coordinates is to first floor the input coordinates
-         * 6) To avoid rounding errors causing us to be unable to generate
-         *    coordinates for the bottom row / left column of pixels we ceil
-         *    the end result
-         */
-        motion->x = ceil(floor(motion->x) * sx);
-        motion->y = ceil(floor(motion->y) * sy);
-    } else {
-        motion->x -= d->mx;
-        motion->y -= d->my;
-    }
+    spicex_transform_input (display, motion->x, motion->y, &x, &y);
 
     switch (d->mouse_mode) {
     case SPICE_MOUSE_MODE_CLIENT:
-        if (motion->x >= 0 && motion->x < d->area.width &&
-            motion->y >= 0 && motion->y < d->area.height) {
+        if (x >= 0 && x < d->area.width &&
+            y >= 0 && y < d->area.height) {
             spice_inputs_position(d->inputs,
-                                  motion->x + d->area.x, motion->y + d->area.y,
+                                  x + d->area.x, y + d->area.y,
                                   d->channel_id,
                                   button_mask_gdk_to_spice(motion->state));
         }
         break;
     case SPICE_MOUSE_MODE_SERVER:
         if (d->mouse_grab_active) {
-            gint dx = d->mouse_last_x != -1 ? motion->x - d->mouse_last_x : 0;
-            gint dy = d->mouse_last_y != -1 ? motion->y - d->mouse_last_y : 0;
+            gint dx = d->mouse_last_x != -1 ? x - d->mouse_last_x : 0;
+            gint dy = d->mouse_last_y != -1 ? y - d->mouse_last_y : 0;
 
             spice_inputs_motion(d->inputs, dx, dy,
                                 button_mask_gdk_to_spice(motion->state));
 
-            d->mouse_last_x = motion->x;
-            d->mouse_last_y = motion->y;
+            d->mouse_last_x = x;
+            d->mouse_last_y = y;
             mouse_wrap(display, motion);
         }
         break;
@@ -1341,6 +1432,7 @@ static gboolean button_event(GtkWidget *widget, GdkEventButton *button)
 {
     SpiceDisplay *display = SPICE_DISPLAY(widget);
     SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(display);
+    int x, y;
 
     SPICE_DEBUG("%s %s: button %d, state 0x%x", __FUNCTION__,
             button->type == GDK_BUTTON_PRESS ? "press" : "release",
@@ -1349,16 +1441,12 @@ static gboolean button_event(GtkWidget *widget, GdkEventButton *button)
     if (d->disable_inputs)
         return true;
 
-    if (!spicex_is_scaled(display)
-        && d->mouse_mode == SPICE_MOUSE_MODE_CLIENT) {
-        gint x, y;
-
+    spicex_transform_input (display, button->x, button->y, &x, &y);
+    if ((x < 0 || x >= d->area.width ||
+         y < 0 || y >= d->area.height) &&
+        d->mouse_mode == SPICE_MOUSE_MODE_CLIENT) {
         /* rule out clicks in outside region */
-        gtk_widget_get_pointer (widget, &x, &y);
-        x -= d->mx;
-        y -= d->my;
-        if (!(x >= 0 && x < d->area.width && y >= 0 && y < d->area.height))
-            return true;
+        return true;
     }
 
     gtk_widget_grab_focus(widget);
@@ -1415,6 +1503,21 @@ static gboolean configure_event(GtkWidget *widget, GdkEventConfigure *conf)
     return true;
 }
 
+static void realize(GtkWidget *widget)
+{
+    GTK_WIDGET_CLASS(spice_display_parent_class)->realize(widget);
+
+    spicex_image_create(SPICE_DISPLAY(widget));
+}
+
+static void unrealize(GtkWidget *widget)
+{
+    spicex_image_destroy(SPICE_DISPLAY(widget));
+
+    GTK_WIDGET_CLASS(spice_display_parent_class)->unrealize(widget);
+}
+
+
 /* ---------------------------------------------------------------- */
 
 static void spice_display_class_init(SpiceDisplayClass *klass)
@@ -1438,6 +1541,8 @@ static void spice_display_class_init(SpiceDisplayClass *klass)
     gtkwidget_class->button_release_event = button_event;
     gtkwidget_class->configure_event = configure_event;
     gtkwidget_class->scroll_event = scroll_event;
+    gtkwidget_class->realize = realize;
+    gtkwidget_class->unrealize = unrealize;
 
     gobject_class->constructor = spice_display_constructor;
     gobject_class->dispose = spice_display_dispose;
@@ -1547,11 +1652,47 @@ static void spice_display_class_init(SpiceDisplayClass *klass)
         (gobject_class, PROP_SCALING,
          g_param_spec_boolean("scaling", "Scaling",
                               "Whether we should use scaling",
+                              TRUE,
+                              G_PARAM_READWRITE |
+                              G_PARAM_CONSTRUCT |
+                              G_PARAM_STATIC_STRINGS));
+
+    /**
+     * SpiceDisplay:only-downscale:
+     *
+     * If scaling, only scale down, never up.
+     *
+     * Since: 0.14
+     **/
+    g_object_class_install_property
+        (gobject_class, PROP_ONLY_DOWNSCALE,
+         g_param_spec_boolean("only-downscale", "Only Downscale",
+                              "If scaling, only scale down, never up",
                               FALSE,
                               G_PARAM_READWRITE |
                               G_PARAM_CONSTRUCT |
                               G_PARAM_STATIC_STRINGS));
 
+    /**
+     * SpiceDisplay:keypress-delay:
+     *
+     * Delay in ms of non-modifiers key press events. If the key is
+     * released before this delay, a single press & release event is
+     * sent to the server. If the key is pressed longer than the
+     * keypress-delay, the server will receive the delayed press
+     * event, and a following release event when the key is released.
+     *
+     * Since: 0.13
+     **/
+    g_object_class_install_property
+        (gobject_class, PROP_KEYPRESS_DELAY,
+         g_param_spec_uint("keypress-delay", "Keypress delay",
+                           "Keypress delay",
+                           0, G_MAXUINT, 100,
+                           G_PARAM_READWRITE |
+                           G_PARAM_CONSTRUCT |
+                           G_PARAM_STATIC_STRINGS));
+
     /**
      * SpiceDisplay:disable-inputs:
      *
@@ -1714,7 +1855,8 @@ static void update_area(SpiceDisplay *display,
 
     spicex_image_destroy(display);
     d->area = area;
-    spicex_image_create(display);
+    if (gtk_widget_get_realized(GTK_WIDGET(display)))
+        spicex_image_create(display);
     update_size_request(display);
 
     set_monitor_ready(display, true);
@@ -1757,6 +1899,9 @@ static void invalidate(SpiceChannel *channel,
 {
     SpiceDisplay *display = data;
     SpiceDisplayPrivate *d = SPICE_DISPLAY_GET_PRIVATE(display);
+    int display_x, display_y;
+    int x1, y1, x2, y2;
+    double s;
     GdkRectangle rect = {
         .x = x,
         .y = y,
@@ -1773,14 +1918,18 @@ static void invalidate(SpiceChannel *channel,
     if (d->convert)
         do_color_convert(display, &rect);
 
-    x = rect.x - d->area.x;
-    y = rect.y - d->area.y;
-    w = rect.width;
-    h = rect.height;
+    spice_display_get_scaling(display, &s,
+                              &display_x, &display_y,
+                              NULL, NULL);
+
+    x1 = floor ((rect.x - d->area.x) * s);
+    y1 = floor ((rect.y - d->area.y) * s);
+    x2 = ceil ((rect.x - d->area.x + rect.width) * s);
+    y2 = ceil ((rect.y - d->area.y + rect.height) * s);
 
-    spicex_image_invalidate(display, &x, &y, &w, &h);
     gtk_widget_queue_draw_area(GTK_WIDGET(display),
-                               x, y, w, h);
+                               display_x + x1, display_y + y1,
+                               x2 - x1, y2-y1);
 }
 
 static void mark(SpiceDisplay *display, gint mark)
@@ -1856,39 +2005,77 @@ static void cursor_hide(SpiceCursorChannel *channel, gpointer data)
 }
 
 G_GNUC_INTERNAL
-void spice_display_get_scaling(SpiceDisplay *display, double *sx, double *sy)
+void spice_display_get_scaling(SpiceDisplay *display,
+                               double *s_out,
+                               int *x_out, int *y_out,
+                               int *w_out, int *h_out)
 {
     SpiceDisplayPrivate *d = display->priv;
     int fbw = d->area.width, fbh = d->area.height;
     int ww, wh;
+    int x, y, w, h;
+    double s;
 
-    if (!spicex_is_scaled(display) || !gtk_widget_get_window(GTK_WIDGET(display))) {
-        *sx = 1.0;
-        *sy = 1.0;
-        return;
+    if (gtk_widget_get_realized (GTK_WIDGET(display)))
+        gdk_drawable_get_size(gtk_widget_get_window(GTK_WIDGET(display)), &ww, &wh);
+    else {
+        ww = fbw;
+        wh = fbh;
     }
 
-    gdk_drawable_get_size(gtk_widget_get_window(GTK_WIDGET(display)), &ww, &wh);
+    if (!spicex_is_scaled(display)) {
+        s = 1.0;
+        x = 0;
+        y = 0;
+        if (ww > d->area.width)
+            x = (ww - d->area.width) / 2;
+        if (wh > d->area.height)
+            y = (wh - d->area.height) / 2;
+        w = fbw;
+        h = fbh;
+    } else {
+        s = MIN ((double)ww / (double)fbw, (double)wh / (double)fbh);
 
-    *sx = (double)ww / (double)fbw;
-    *sy = (double)wh / (double)fbh;
+        if (d->only_downscale && s >= 1.0)
+            s = 1.0;
+
+        /* Round to int size */
+        w = floor (fbw * s + 0.5);
+        h = floor (fbh * s + 0.5);
+
+        /* Center the display */
+        x = (ww - w) / 2;
+        y = (wh - h) / 2;
+    }
+
+    if (s_out)
+        *s_out = s;
+    if (w_out)
+        *w_out = w;
+    if (h_out)
+        *h_out = h;
+    if (x_out)
+        *x_out = x;
+    if (y_out)
+        *y_out = y;
 }
 
 static void cursor_invalidate(SpiceDisplay *display)
 {
     SpiceDisplayPrivate *d = display->priv;
-    double sx, sy;
+    double s;
+    int x, y;
 
     if (d->mouse_pixbuf == NULL)
         return;
 
-    spice_display_get_scaling(display, &sx, &sy);
+    spice_display_get_scaling(display, &s, &x, &y, NULL, NULL);
 
     gtk_widget_queue_draw_area(GTK_WIDGET(display),
-                               (d->mouse_guest_x + d->mx - d->mouse_hotspot.x - d->area.x) * sx,
-                               (d->mouse_guest_y + d->my - d->mouse_hotspot.y - d->area.y) * sy,
-                               gdk_pixbuf_get_width(d->mouse_pixbuf) * sx,
-                               gdk_pixbuf_get_height(d->mouse_pixbuf) * sy);
+                               floor ((d->mouse_guest_x - d->mouse_hotspot.x - d->area.x) * s) + x,
+                               floor ((d->mouse_guest_y - d->mouse_hotspot.y - d->area.y) * s) + y,
+                               ceil (gdk_pixbuf_get_width(d->mouse_pixbuf) * s),
+                               ceil (gdk_pixbuf_get_height(d->mouse_pixbuf) * s));
 }
 
 static void cursor_move(SpiceCursorChannel *channel, gint x, gint y, gpointer data)
